package dense.randGAME.levelgen;

import java.awt.Color;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Random;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

public class PerlinNoise {
	public PerlinNoise(){
		
	}
	
	public static float[][] GenerateSmoothNoise(float[][] baseNoise, int octave)
	{
	   int width = baseNoise.length;
	   int height = baseNoise[0].length;
	 
	   float[][] smoothNoise = GetEmptyArray(width, height);
	 
	   int samplePeriod = 1 << octave; // calculates 2 ^ k
	   float sampleFrequency = 1.0f / samplePeriod;
	 
	   for (int i = 0; i < width; i++)
	   {
	      //calculate the horizontal sampling indices
	      int sample_i0 = (i / samplePeriod) * samplePeriod;
	      int sample_i1 = (sample_i0 + samplePeriod) % width; //wrap around
	      float horizontal_blend = (i - sample_i0) * sampleFrequency;
	 
	      for (int j = 0; j < height; j++)
	      {
	         //calculate the vertical sampling indices
	         int sample_j0 = (j / samplePeriod) * samplePeriod;
	         int sample_j1 = (sample_j0 + samplePeriod) % height; //wrap around
	         float vertical_blend = (j - sample_j0) * sampleFrequency;
	 
	         //blend the top two corners
	         float top = Interpolate(baseNoise[sample_i0][sample_j0],
	            baseNoise[sample_i1][sample_j0], horizontal_blend);
	 
	         //blend the bottom two corners
	         float bottom = Interpolate(baseNoise[sample_i0][sample_j1],
	            baseNoise[sample_i1][sample_j1], horizontal_blend);
	 
	         //final blend
	         smoothNoise[i][j] = Interpolate(top, bottom, vertical_blend);
	      }
	   }
	 
	   return smoothNoise;
	}
	
	private static float Interpolate(float x0, float x1, float alpha)
	{
	   return x0 * (1 - alpha) + alpha * x1;
	}
	
	public static float[][] GeneratePerlinNoise(float[][] baseNoise, int octaveCount)
	{
	   int width = baseNoise.length;
	   int height = baseNoise[0].length;
	 
	   float[][][] smoothNoise = new float[octaveCount][][]; //an array of 2D arrays containing
	 
	   float persistance = 0.2f;
	 
	   //generate smooth noise
	   for (int i = 0; i < octaveCount; i++)
	   {
	       smoothNoise[i] = GenerateSmoothNoise(baseNoise, i);
	   }
	 
	    float[][] perlinNoise = GetEmptyArray(width, height);
	    float amplitude = 1.0f;
	    float totalAmplitude = 0.0f;
	 
	    //blend noise together
	    for (int octave = octaveCount - 1; octave >= 0; octave--)
	    {
	       amplitude *= persistance;
	       totalAmplitude += amplitude;
	 
	       for (int i = 0; i < width; i++)
	       {
	          for (int j = 0; j < height; j++)
	          {
	             perlinNoise[i][j] += smoothNoise[octave][i][j] * amplitude;
	          }
	       }
	    }
	 
	   //normalisation
	   for (int i = 0; i < width; i++)
	   {
	      for (int j = 0; j < height; j++)
	      {
	         perlinNoise[i][j] /= totalAmplitude;
	      }
	   }
	 
	   return perlinNoise;
	}

	private static float[][] GetEmptyArray(int width, int height) {
		float[][] array = new float[width][height];
		return array;
	}
	
	public static float[][] GenerateWhiteNoise(int width, int height)
	{
	    Random random = new Random(); //Seed to 0 for testing
	    float[][] noise = new float[width][height];
	 
	    for (int i = 0; i < width; i++)
	    {
	        for (int j = 0; j < height; j++)
	        {
	            noise[i][j] = (float)random.nextDouble() % 1;
	        }
	    }
	 
	    return noise;
	}
	
	private static Color GetColor(Color gradientStart, Color gradientEnd, float t)
	{
	    float u = 1 - t;
	    
	    
	 
	    Color color = new Color( 
	    		(int)(gradientStart.getRed() * u + gradientEnd.getRed() * t), 
	    		(int)(gradientStart.getGreen() * u + gradientEnd.getGreen() * t), 
	    		(int)(gradientStart.getBlue() * u + gradientEnd.getBlue() * t));
	 
	    return color;
	}
	 
	public static Color[][] MapGradient(Color gradientStart, Color gradientEnd, float[][] perlinNoise)
	{
	   int width = perlinNoise.length;
	   int height = perlinNoise[0].length;
	 
	   Color[][] image = new Color[width][height];
	 
	   for (int i = 0; i < width; i++)
	   {
	      for (int j = 0; j < height; j++)
	      {
	         image[i][j] = GetColor(gradientStart, gradientEnd, perlinNoise[i][j]);
	      }
	   }
	 
	   return image;
	}
	
	public static void main(String args[]){
		PerlinNoise noise = new PerlinNoise();
		int w = 40;
		int h = 40;
		Color[][] cimage = noise.MapGradient(Color.green, Color.blue, noise.GeneratePerlinNoise(noise.GenerateWhiteNoise(w, h),5));
		BufferedImage img = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
		int[] pixels = new int[w * h];
		int x = 0;
		for(int i = 0; i < w; i++){
			for(int j = 0; j < h; j++){
				pixels[x] = cimage[i][j].getRGB();
				x++;
			}
		}	
		
		img.setRGB(0, 0, w, h, pixels, 0, w);
		JOptionPane.showMessageDialog(null, null, "Another", JOptionPane.YES_NO_OPTION, new ImageIcon(img.getScaledInstance(w * 4, h * 4, Image.SCALE_AREA_AVERAGING)));
	}
}
